/*
   ServoTest Sample sketch
 
 An example code for testing each servo channels.
 Use SoftwareSerial pin 2 (RX) and 3 (TX).
 
 ServoDriver to gizDuino
 TX  --> pin2 (RX)
 RX  --> pin3 (TX)
 GND --> GND
 PWR --> 5V supply*
 
 Note*: you can use 5V supply on the gizduino required
 only 1-2 servo like SG-90.
 If 3 or more servo SG-90 to MG995 etc., with high
 current/torque,please use SoftStarter to limit/
 control the current on each servo motors
 and seperate the supply.
 
 SoftStarter kit : (Optional) http://goo.gl/Ak9x17
 Adaptor: (Optional) http://goo.gl/AbxLuQ
 
 Codes by
 e-Gizmo Mechatronix Central
 November 2, 2018
 http://www.e-gizmo.net/oc
 
 */

#include "eGizmo_16channelServo.h"
//SoftwareSerial is already included

SoftwareSerial softSerial(2,3);
eGizmo_16channelServo servoDriver(&softSerial);

//To use hardware Serial pins, use this instead
//eGizmo_16channelServo servoDriver(&Serial);

void setup() {
  // put your setup code here, to run once
  servoDriver.begin(9600);
  servoDriver.debugMode(ENABLE);
  Serial.begin(9600);
  Serial.println(F("Code Start!"));
}//Setup

void loop() {

  for(int i=0; i<16; i++){          // Servo Channels from 0 to 16
    servoDriver.moveServo(i, 160);  // Send 160 degrees to servos
    Serial.print(F("Waiting for "));
    Serial.print(i);
    Serial.println(F("..."));
    servoDriver.debugMode(DISABLE);
    while(servoDriver.isServoActive(i)){
    }	//dynamic delay
    servoDriver.debugMode(ENABLE);
    Serial.println(F("Done!"));
  }//for
  delay(1000);
  for(int i=0; i<16; i++){ 
    servoDriver.moveServo(i, 10);
    Serial.print(F("Waiting for "));
    Serial.print(i);
    Serial.println(F("..."));
    servoDriver.debugMode(DISABLE);
    while(servoDriver.isServoActive(i)){
    }	//dynamic delay
    servoDriver.debugMode(ENABLE);
    Serial.println(F("Done!"));
  }//for
  delay(1000);

  //Servo controls with slew option
  for(int i=0; i<16; i++){
    servoDriver.moveServo(i, 160, 10);
    Serial.print(F("Waiting for "));
    Serial.print(i);
    Serial.println(F("..."));
    servoDriver.debugMode(DISABLE);
    while(servoDriver.isServoActive(i)){
    }	//dynamic delay
    servoDriver.debugMode(ENABLE);
  }//for
  delay(1000);
  for(int i=0; i<16; i++){
    servoDriver.moveServo(i, 10, 10);
    Serial.print(F("Waiting for "));
    Serial.print(i);
    Serial.println(F("..."));
    servoDriver.debugMode(DISABLE);
    while(servoDriver.isServoActive(i)){
    }	//dynamic delay
    servoDriver.debugMode(ENABLE);
  }//for
  delay(1000);
}//Loop
